@@ -11,7 +11,8 @@
 # the open interval to the *right* of the current lambda_k.
 
 dualpathWideSparse <- function(y, D, x=NULL, approx=FALSE, maxsteps=2000,
-                               minlam=0, tol=1e-11, verbose=FALSE, object=NULL) { 
+                               minlam=0, rtol=1e-7, btol=1e-7, verbose=FALSE,
+                               object=NULL) { 
   # If we are starting a new path
   if (is.null(object)) {
     m = nrow(D)
@@ -111,6 +112,7 @@ dualpathWideSparse <- function(y, D, x=NULL, approx=FALSE, maxsteps=2000,
 
         # Make sure none of the hitting times are larger
         # than the current lambda (precision issue)
+        hits[hits>lams[k-1]+btol] = 0
         hits[hits>lams[k-1]] = lams[k-1]
         
         ihit = which.max(hits)
@@ -136,6 +138,7 @@ dualpathWideSparse <- function(y, D, x=NULL, approx=FALSE, maxsteps=2000,
         
         # Make sure none of the leaving times are larger
         # than the current lambda (precision issue)
+        leaves[leaves>lams[k-1]+btol] = 0
         leaves[leaves>lams[k-1]] = lams[k-1]
         
         ileave = which.max(leaves)
@@ -215,7 +218,7 @@ dualpathWideSparse <- function(y, D, x=NULL, approx=FALSE, maxsteps=2000,
   # Save needed elements for continuing the path
   pathobjs = list(type="wide.sparse", r=r, B=B, I=I, Q1=NA, approx=approx,
     Q2=NA, k=k, df=df, D1=D1, D2=D2, Ds=Ds, ihit=ihit, m=m, n=n, q=q, h=h,
-    R=NA, q0=NA, tol=tol, s=s, y=y)
+    R=NA, q0=NA, rtol=rtol, btol=btol, s=s, y=y)
   
   # If we reached the maximum number of steps
   if (k>maxsteps) {
